/* History:
 *   Created: Tue Apr 07 1998
 *   Maintainer: 0rfelyus (temporarily)
 *
 *   Mon Dec 21 01:20:07 CET 1998 vitas
 *   added TR_vec TR_mat and lot of matrix and vec functions
 *   removed all 0rf. functions :)
 *  
 */
 
#ifndef __MATRIX_H_
#define __MATRIX_H_

#include <math.h>
#include <string.h>
#include <stdio.h>


/* Defines */

#define CPYVEC(d, s)  memcpy((void*)(d), (void*)(s),  4 * sizeof(double));
#define CPYMAT(d, s)  memcpy((void*)(d), (void*)(s), 16 * sizeof(double));

#define ZEROVEC(v)    memset((void*)(v), 0,  4 * sizeof(double))
#define ZEROMAT(m)    memset((void*)(m), 0, 16 * sizeof(double))

/* Types */

/* Functions */


#ifdef __cplusplus
extern "C" {
#endif

extern double mat_identity[16];
extern double vec_e1[4];
extern double vec_e2[4];
extern double vec_e3[4];
extern double vec_zero[4];

void mat_mul_mat(double *res, double *mat1, double *mat2);
void mat_add_mat(double *res, double *mat1, double *mat2);
void mat_sub_mat(double *res, double *mat1, double *mat2);
void mat_neg(double *res, double *mat);

int mat_inverse(double *res, double *mat);
void mat_mul_double(double *r, double *mat, double a);

__inline void mat_load_identity(double *res)
  { CPYMAT(res, mat_identity); }

void mat_load_rot(double *res, double *vec, double angle);
void mat_load_rotx(double *res, double angle);
void mat_load_roty(double *res, double angle);
void mat_load_rotz(double *res, double angle);
void mat_load_rotxg(double *res, double sinangle, double cosangle);
void mat_load_rotyg(double *res, double sinangle, double cosangle);
void mat_load_rotzg(double *res, double sinangle, double cosangle);

void mat_load_scale(double  *res, double *vec);
void mat_neg(double *res, double *m);
void mat_mul_double(double *res, double *m, double a);
void mat_transp(double *res, double *m);
int mat_is_identity(const double *mat);
void mat_get_rot(double *res, double*vec, double angle);

void mat_mul_vec(double *res, double *vec, double *mat);
void vec_mul_mat(double *res, double *mat, double *vec);

__inline void vec_mul_double(double *r, double *v ,double a)
  { r[0] = a * v[0]; r[1] = a * v[1]; r[2] = a * v[2];}

__inline double vec_abs(double *v)
  { return  sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);}

__inline void vec_add_vec(double *r, double *u, double *v)
  { r[0] = u[0] + v[0]; r[1] = u[1] + v[1]; r[2] = u[2] + v[2]; r[3]= 1; }

__inline void vec_sub_vec(double *r, double *u, double *v)
  { r[0] = u[0] - v[0]; r[1] = u[1] - v[1]; r[2] = u[2] - v[2]; r[3]= 1; }

__inline void vec_neg(double *r, double *v)
  { r[0] =  - v[0]; r[1] =  - v[1]; r[2] = - v[2]; }

__inline double vec_mul_vec(double *u, double *v)
  { return  u[0] * v[0] - u[1] * v[1] + u[2] * v[2]; }

void vec_times_vec(double *r, double *u,  double *v);

void vec_norm(double *res, double *v);

#ifdef __cplusplus
}
#endif


/* classes */

#ifdef __cplusplus


class t_mat;
class t_vec
{
// ale co delat kdyz to neni to spravne pole (vyjimka?)
private:
  void init(){};
public:
  double d[4];

public:

  t_vec(double x = 0, double y = 0, double  z = 0, double w = 1.0)
    {init(); d[0] = x; d[1] =  y; d[2] =  z; d[3] = w;};

  t_vec(double *v)
    {init(); CPYVEC(d, v);};

  double abs()
    {return vec_abs(d);}

  void zero()
    { CPYVEC(d, vec_zero);};

  void load_e1()
    { CPYVEC(d, vec_e1);};

  void load_e2()
    { CPYVEC(d, vec_e2);};

  void load_e3()
    { CPYVEC(d, vec_e3);};

  t_vec& norm()
    {vec_norm(d, d); return *this;};  

   double&operator[](int i)
    {return d[i & 3];};

  t_vec& inc(t_vec v)
   {
    d[0]+= v.d[0];
    d[1]+= v.d[1];
    d[2]+= v.d[2];
    return *this;
  }; /* inc */
  
  t_vec& dec(t_vec v)
   {
    d[0]-= v.d[0];
    d[1]-= v.d[1];
    d[2]-= v.d[2];
    return *this;
  };/* dec */

  void print(char *s ="")
    { printf("%s(%f %f %f %f)\n", s, d[0], d[1], d[2], d[3]); };

  double x () const { return d[0]; };
  double y () const { return d[1]; };
  double z () const { return d[2]; };
  double w () const { return d[3]; };

  double& x() { return d[0]; };
  double& y() { return d[1]; };
  double& z() { return d[2]; };
  double& w() { return d[3]; };

  double& x(double val) { return d[0] = val; };
  double& y(double val) { return d[1] = val; };
  double& z(double val) { return d[2] = val; };
  double& w(double val) { return d[3] = val; };

};/* class vec */

class t_mat
{
//FIXME: chtelo by to konstruktor mat(TR&)
private:
  void init(){};
public:
  /*
   * note: matrix indexes are: 
   *   ,         .
   *  | 0 4  8 12 |
   *  | 1 5  9 13 |
   *  | 2 6 10 14 |
   *  | 3 7 11 15 |
   *  `           '
   */
  double d[16];
public:
  t_mat()
    { init(); };

  t_mat(
       double a0, double a1, double a2, double a3,
       double a4=0, double a5=0, double a6=0, double a7=0, 
       double a8=0, double a9=0, double aa=0, double ab=0,
       double ac = 0.0, double ad = 0.0,
       double ae  =0.0, double af = 1.0)
  {
    init();
    d[ 0]= a0; d[ 1]= a1; d[ 2]= a2; d[ 3]= a3; 
    d[ 4]= a4; d[ 5]= a5; d[ 6]= a6; d[ 7]= a7; 
    d[ 8]= a8; d[ 9]= a9; d[10]= aa; d[11]= ab; 
    d[12]= ac; d[13]= ad; d[14]= ae; d[15]= af; 
  };

  t_mat( double *m)
    {init(); CPYMAT(d, m);};

  void load_zero()
   { ZEROMAT(d);};

  void load_identity()
   { ZEROMAT(d); d[ 0] = d[ 5] = d[10] = d[15] = 1; };


  void load_rot(t_vec& v, double fi)
   { mat_load_rot(d, v.d, fi);};

  void load_rotx(double fi)
   { mat_load_rotx(d, fi);};

  void load_roty(double fi)
   { mat_load_roty(d, fi);};

  void load_rotz(double fi)
   { mat_load_rotz(d, fi);};

  void load_scale(t_vec v)
   { ZEROMAT(d); d[0]=v.x(); d[5]=v.y(); d[10]=v.z(); d[15]=1.0;};

  void load_trans(t_vec v)
   { load_identity(); d[3]=v.x(); d[7]=v.y(); d[11]=v.z();};

  int invese()
   { return mat_inverse(d,d); }

  t_mat& inc(t_mat& m)
   { mat_add_mat(d, d, m.d); return *this; };

  t_mat& dec(t_mat& m)
   { mat_sub_mat(d, d, m.d); return *this; };
  
  double& operator[](int i)
   { return d[i & 0xf];};

  void print(char *s ="") {
	printf("(%f %f %f %f\n", d[0], d[1], d[2], d[3]);
	printf(" %f %f %f %f\n", d[4], d[5], d[6], d[7]);
	printf(" %f %f %f %f\n", d[8], d[9], d[10], d[11]);
	printf(" %f %f %f %f)\n", d[12], d[13], d[14], d[15]);
	};
};/* class mat */

/* Public variables */


/* inline oprerators vec */

inline t_vec operator+(t_vec u, t_vec v)
  { return t_vec( u[0] + v[0], u[1] + v[1], u[2] + v[2] ); }

inline t_vec operator-(t_vec u, t_vec v)
  { return t_vec( u[0] - v[0], u[1] - v[1], u[2] - v[2]); }

inline t_vec operator-(t_vec v)
  { return t_vec( - v[0], - v[1],  - v[2]); }

inline double operator*(t_vec u, t_vec v)
  { return u[0] * v[0] + u[1] * v[1] + u[2] * v[2]; }

inline t_vec operator*(double a, t_vec v)
  { return t_vec( a * v[0], a *  v[1],  a * v[2]); }

inline t_vec operator*(t_vec &v, double a)
  { return t_vec( a * v.d[0], a *  v.d[1],  a * v.d[2]); }

inline t_vec operator/(t_vec &v, double a)
  { return t_vec( v.d[0] / a, v.d[1] / a,  v.d[2] / a); }

inline t_vec operator*(t_vec& v, t_mat& m)
  { t_vec ret; vec_mul_mat(ret.d, v.d, m.d); return ret; }

inline t_vec operator*(t_mat& m, t_vec& v)
  { t_vec ret; mat_mul_vec(ret.d, m.d, v.d); return ret; }

/*
 * vec product 
 */


inline t_vec operator^(t_vec  u,  t_vec  v)
  { return t_vec( u[1] * v[2] - u[2] * v[1],
                 u[2] * v[0] - u[0] * v[2],
                 u[0] * v[1] - u[1] * v[0]);}
/* inline oprerators mat */

inline t_mat operator+(t_mat&a, t_mat&b)
  { t_mat ret; mat_add_mat(ret.d, a.d, b.d); return ret; }

inline t_mat operator-(t_mat& a, t_mat& b)
  { t_mat ret; mat_add_mat(ret.d, a.d, b.d); return ret; }

inline const t_mat operator*(t_mat& a, t_mat& b)
  { t_mat ret; mat_mul_mat(ret.d, a.d, b.d); return ret; }

inline t_mat operator*(double a, t_mat& b)
  { t_mat ret; mat_mul_double(ret.d, b.d, a); return ret; }

inline t_mat operator*(t_mat& b, double a)
  { t_mat ret; mat_mul_double(ret.d, b.d, a); return ret; }

inline t_mat operator-(t_mat& a)
  { t_mat ret; mat_neg(ret.d, a.d); return ret; }

inline t_mat operator!(t_mat& a)
  { t_mat ret; mat_inverse(ret.d, a.d); return ret; }

/* inline oprerators mat-vec-double */

/* Public functions */ 

#endif

#endif
