#include "mat.h"
#include "obal.h"

#include <math.h>
#include <vector>

using namespace std;

t_poly_container polys;

t_vec
car2pol(t_vec &v) {
	t_vec ret;

	ret.x() = v.abs();
	ret.y() = atan2(v.x(),v.y());
	ret.z() = 0; /*FIXME*/
	return ret;
}

void
t_poly_container::def(string w, string typ, string args)
{
	if( end() != find(w) ) {
		err("poly already defined: %s", w.c_str());
	}

	if( typ == "n" ) {
		(*this)[w]= new t_nuhelnik(args);
		return;
	}
	if( typ == "l" ) {
		(*this)[w]= new t_lichobeznik(args);
		return;
	}
	err("unknown poly type:%s", typ.c_str());
} /* t_poly_container::def */

void
t_poly_container::con(string w1, int n1, string w2, int n2)
{
	if( end() == find(w1) ) {
		err("poly not defined: %s", w1.c_str());
	}

	if( end() == find(w2) ) {
		err("poly not defined: %s", w2.c_str());
	}
	con2( (*this)[w1], n1, (*this)[w2], n2);
} /* t_poly_container::def */

void
t_poly_container::con2(t_poly *to, int to_n, t_poly *from, int from_n = 0)
{
	int to_siz = to->verts.size();
	t_vec v1 = to->verts[   to_n       % to_siz ];
	t_vec v2 = to->verts[ ( to_n + 1 ) % to_siz ];
	
	from->join(from_n, v1, v2);
}  /* t_poly_container::con2 */
void
t_poly_container::print(ostream &o)
{
	iterator i;

	o <<"prologues:=1;\n\nbeginfig(1)\nu:=2cm;\n";
	for(i=begin(); i != end(); i++) {
		(*i).second->print(o);
		if(want_numbers) {
			(*i).second->printcom(o, (*i).first);
		}
	}
	cout <<"endfig;\nend\n";
} /* t_poly_container::print */

void
t_poly::join(int idx, t_vec v1, t_vec v2)
{
	t_mat m1, m2, m3, m4, m;
	
	t_vec u1 = verts[idx];
	t_vec u2 = verts[(idx-1+verts.size())%verts.size()];

	t_vec du = u2 - u1;
	t_vec dv = v2 - v1;

	double absu = du.abs();
	double absv = dv.abs();
	double sc=absv/absu;

	double ang = car2pol(dv).y() - car2pol(du).y();

	/* je treba `m' tak, aby v1 == u1*m , a v2 == u2 * m */

	m1.load_trans(-u1);
	m2.load_rotz(ang);
	m3.load_scale(t_vec(sc,sc,sc));
	m4.load_trans(v1);

#if 0
	m1.print();

	printf("m2:\n");
	m2.print();

	printf("m3:\n");
	m3.print();

	printf("m4:\n");
	m4.print();

	m = m1 * m2;
	printf("m1*m2:\n");

	m.print();
	m = m * m3;

	printf("m1*m2*m3:\n");
	m.print();

	m = m * m4;
	printf("m1*m2*m3*m4:\n");

	m.print();
#else
	m = m1 * m2;
	m = m * m3;
	m = m * m4;
#endif
	
	aply( m );
	
	
} /* join*/


typedef vector<t_nuhelnik> t_nuhelniky;
/*
 * con(15, 16, 1):
 *
 * 1  0 1  0
 *  +--+--+
 *  |16|15|
 *  +--+--+
 * 2  3 2  3
 *
 */
t_nuhelniky nu;

int
con(int from, int to, int from_vert, int to_vert = 0)
{
	t_vec v1 = nu[from].verts[from_vert];
	t_vec v2 = nu[from].verts[(from_vert+1)%nu[from].verts.size()];
	
	nu[to].join(to_vert, v1, v2);
} 

void
add(int i,int n)
{
	nu[i] = t_nuhelnik(n);
}

void
print()
{
	t_nuhelniky::iterator i;

	cout <<"prologues:=1;\n\nbeginfig(1)\nu:=2cm;\n";
	for(i=nu.begin(); i!=nu.end(); i++) {
		(*i).print(cout);
		//(*i).printcom(cout, 'a' + i - nu.begin());
	}
	cout <<"endfig;\nend\n";
}
	

int
main()
{
#if 0
	nu.resize(6);

	add(0,4);
	add(1,4);
	add(2,4);
	add(3,3);
	add(4,3);
	add(5,3);
	
	con(0,1,3);
	con(1,2,1);
	con(2,3,0);
	con(2,4,1);
	con(2,5,2);

	print();

#endif
	t_obal_lex lex;

	lex.parse();
	polys.want_numbers = 1;

	polys.print(cout);
};

