#ifndef __OBAL_H
#define __OBAL_H

#include "mat.h"
#include <vector>
#include <map>
#include <string>
#include <FlexLexer.h>

using namespace std;


class t_poly
{
public:
	t_poly(int n = 0){
		verts.resize(n);
		int i; 
		double a, delta_angle = (M_PI * 2)/n;
		for(i=0, a=-delta_angle/2;i<n;i++,a+=delta_angle) {
			verts[i].x() = cos(a);
			verts[i].y() = sin(a);
		}
	};
	string name;
	typedef vector<t_vec> t_verts;
	t_verts verts;
	int n(){return verts.size();}

	void
	printvec(ostream& o, t_vec v){ 
		o.setf(ios::fixed);
		o.precision(4);
		o << "(" << v.x() << "u," <<
		v.y() <<"u)";
	};

	void join(int idx, t_vec v1, t_vec v2);
	void aply(t_mat m) {
		t_verts::iterator i;
		for(i= verts.begin();i!=verts.end();i++) {
			(*i) = (*i) * m;
		}
	};
	t_vec orto(){
		t_vec ret;
		t_verts::iterator i;
		for(i= verts.begin();i!=verts.end();i++) {
			ret = ret + (*i);
		}
		ret = ret /(double)verts.size();
		return ret;
	};
	ostream& print(ostream &o) {
		t_verts::iterator i;

		o << "draw    ";
		for(i= verts.begin();i!=verts.end();i++) {
			printvec(o,(*i));
			o << "--\n\t";
		}
		o <<"cycle;\n";
		return o;
	};

	ostream& printcom(ostream &o, string com) {
		t_verts::iterator i;
		t_vec cen= orto();

		o << "\n\nlabel(\"" <<com << "\",";
		printvec(o,cen);
		o << ");\n";

		for(i= verts.begin();i!=verts.end();i++) {
			o << "label(\"" << i - verts.begin() << "\",";
			printvec(o, (*i) + (cen - (*i)).norm() * 0.3 );
			o << ");\n";
		}
		o << "\n";
		return o;
	};

}/* class t_poly */;



class t_nuhelnik:public t_poly
{
private:
	void
	init(int n = 0) {
		verts.resize(n);
		int i; 
		double a, delta_angle = (M_PI * 2)/n;
		for(i=0, a=-delta_angle/2;i<n;i++,a+=delta_angle) {
			verts[i].x() = cos(a);
			verts[i].y() = sin(a);
		}
	};
public:
	t_nuhelnik(int n = 0){init(n);};
	t_nuhelnik(string arg){
		int n;
		sscanf(arg.c_str(), "(%i)", &n);
		init(n);
	};
};/* class t_nuhelnik */

class t_lichobeznik:public t_poly
{
private:
	void init(double v, double c) {
		verts.resize(4);
		verts[0] = t_vec(.5,0);
		verts[1] = t_vec(c/2,v);
		verts[2] = t_vec(-c/2,v);
		verts[3] = t_vec(-.5,0);
	};
public:
	t_lichobeznik(string par) {
		double v, c;
		sscanf(par.c_str(), "(%lf,%lf)", &v, &c);
		init(v,c);
	};
	t_lichobeznik(double v, double c) {
		init(v,c);
	};
};/* class t_lichobeznik */

class t_poly_container:public map<string,t_poly*>
{
	
	void con2(t_poly *w1, int n1, t_poly *w2, int n2);
public:
	t_poly_container():want_numbers(0){};
	int want_numbers;
	void def(string w, string typ, string args);
	void con(string w1, int n1, string w2, int n2);
	void print(ostream &o);
};/* class t_poly_container */

extern t_poly_container polys;

enum e_obal {
	tag_id = 1,
	tag_par,
	tag_def,
	tag_con,
	tag_num,
	tag_eq,
	tag_semi,
	tag_lt,
	tag_dot,
};

class t_obal_lex:public yyFlexLexer
{
public:
	int unnamed_no;
	t_obal_lex( istream *is=NULL, ostream *os=NULL)
		:yyFlexLexer(is,os),unnamed_no(0){};

	void parse();
	void lex_expr(e_obal end);
private:
        void LexerError(const char *s);
	void lex_con();
	void lex_def();
}; /* class t_aalex */


#ifdef __cplusplus
extern "C" {
#endif

void err(char *fmt, ...);
void db(char *fmt, ...);
void war(char *fmt, ...);

void errl(char *func_name, int lineno, char *fmt, ...);
void warl(char *func_name, int lineno, char *fmt, ...);

#ifdef __cplusplus
}
#endif

#endif __OBAL_H
