%option yylineno
%option c++
%{
#include "obal.h"
#include <stdio.h>
%}

ID	[a-zA-Z][a-zA-Z0-9]*
NUM	[0-9]+
PAR	"("[^")"]*")"
DEF	"def"
CON	"con"
W       [ \t]*

%%
{PAR}	{return tag_par;}
{CON}	{return tag_con;}
{DEF}	{return tag_def;}
{ID}	{return tag_id;}
{NUM}	{return tag_num;}
"="	{return tag_eq;}

";"	{return tag_semi;}
"<"	{return tag_lt;}
"."	{return tag_dot;}

#.*	/* eat comment */
{W}	/* eat white */
\n                      /* eat commas */
.                       { LexerError("unexepted char"); }

%%


void
t_obal_lex::lex_def(){
	int lx;
	string id, poly, par;

	lx = yylex();
	if( lx != tag_id ) {
		LexerError("expected id");
	}
	id = string(YYText());

	lx = yylex();
	if( lx != tag_eq ) {
		LexerError("expected '='");
	}

	lx = yylex();
	if( lx != tag_id ) {
		LexerError("expected poly type");
	}
	poly = string(YYText());

	lx = yylex();
	if( lx != tag_par ) {
		LexerError("expected poly parameters");
	}
	par = string(YYText());
	
	lx = yylex();
	if( lx != tag_semi ) {
		LexerError("expected ';'");
	}

	//cout << "def " << id << " = " << poly << par << ";\n";

	polys.def(id, poly, par);
}/* lex_def */


void
t_obal_lex::lex_expr(e_obal end)
{
	string s, id, poly, par;
	int no;
	int aspon_neco = 0;
	int lx;

	lx = yylex();
	if( lx != tag_id ) {
		LexerError("expected id");
	}

	s = string(YYText());
	
	lx = yylex();
	if( lx == tag_eq ) {
		id = s;

		lx = yylex();
		if( lx != tag_id ) {
			LexerError("expected poly");
		}
		poly = string(YYText());
		lx = yylex();
	}

	if( lx == tag_par ) {
		par = string(YYText());
		lx = yylex();
	}

	if( lx == tag_dot ) {

		lx = yylex();
		if( lx != tag_num ) {
			lx = yylex();
		}
		no = atoi(YYText());
	}
} /* t_obal_lex::lex_expr */





void
t_obal_lex::lex_con(){
	int lx;
	string id1, id2;
	int num1 = 0 , num2 = 0 ;

	lx = yylex();
	if( lx != tag_id ) {
		LexerError("expected id");
	}
	id1 = string(YYText());

	lx = yylex();
	if( lx == tag_dot ) {

		lx = yylex();
		if( lx != tag_num ) {
			LexerError("expected num");
		}
		num1 = atoi(YYText());

		lx = yylex();
	}

	if( lx != tag_lt ) {
		LexerError("expected '<'");
	}

	lx = yylex();
	if( lx != tag_id ) {
		LexerError("expected second id");
	}
	id2 = string(YYText());

	lx = yylex();
	if( lx == tag_dot ) {

		lx = yylex();
		if( lx != tag_num ) {
			LexerError("expected second num");
		}
		num2 = atoi(YYText());

		lx = yylex();
	}

	if( lx != tag_semi ) {
		LexerError("expected ';'");
	}

	//cout << "con " << id1 << "." << num1 << " > " <<
	//	id2  << "." << num2 << ";\n";

	polys.con(id1, num1, id2, num2);
} /* t_obal_lex::lex_con*/

void
t_obal_lex::parse()
{
	int lx;
	while( 0 != (lx = yylex()) ) {
		switch(lx) {
		case tag_def:
			lex_def();
			break;
		case tag_con:
			lex_con();
			break;
		default:
			printf("parse:lx:%i\n", lx);
			LexerError("unexpected tag (ex `def' or `con')");
		}
	}
} /* t_obal_lex::parse */
void
t_obal_lex::LexerError(const char *msg)
{
        war("lex: %s:line %d: yytext[%s]\n", msg, lineno(), YYText());
        exit(0);
} /* aa_err */

int
yywrap()
{
        return 1;
}

void
err(char *fmt, ... )
{
  va_list list;

  va_start(list , fmt);

  vfprintf(stderr, fmt, list);
  fprintf(stderr, "\n");

  va_end;

  exit(0);

}/* err */

void
db(char *fmt, ... )
{
#if 1
  va_list list;

  va_start(list , fmt);

  vfprintf(stderr, fmt, list);
  fprintf(stderr, "\n");

  va_end;
#endif

}/* err */

void
war(char *fmt, ... )
{
  va_list list;

  va_start(list , fmt);

  vfprintf(stderr, fmt, list);
  fprintf(stderr, "\n");

  va_end;

}




