#include "mat.h"
#include <math.h>
#include <vector>

typedef vector<t_vec>  t_vecs;


t_vecs vecs;

double tri_merge(int n)
{
	double alfa;
	double ksi;

	alfa = M_PI * (  1.0 - 2.0 / (double)n );
/*
 *
 * d = 2 sin (alfa / 2);
 * D = sin( alfa );
 * sin(ksi) = d / (2 *D);
 */

/*
 * ksi = 2.0 * asin(sin(alfa/2.0)/sin(alfa));
 * which is the same as:
 */
	ksi = 2.0 * asin( .5 / cos(.5 * alfa ));
	

	return ksi;

} /* tri_merge */


void
do_nuhelnik(int n)
{
	int i;
	double delta_angle = (M_PI * 2.0) / (double)n;
	double angle;
	double r = .5 / sin(delta_angle/2);
	for(i=0,angle=delta_angle/2;i<n;i++,angle+=delta_angle){
		vecs[i].x() = r * cos(angle);
		vecs[i].y() = r * sin(angle);
		vecs[i].z() = 0;
	}
} /* do_nuhelnik */ 

void
otocka(int d, int s, int o1, int o2, double angle) 
{
	t_mat m;
	
	t_vec ss = vecs[s] - vecs[o1];
	t_vec dd;
	t_vec delta_o = vecs[o2] - vecs[o1];

	//delta_o. print();
	m.load_rot(delta_o, - angle);
	//m.print();


	dd = ss * m;
	vecs[d] = dd + vecs[o1];
} /* otocka */

void
do_6()
{
	double a = tri_merge(4);
	do_nuhelnik(4);
	otocka(4, 3, 0, 1, a);
	otocka(5, 0, 1, 2, a);
	otocka(6, 1, 2, 3, a);
	otocka(7, 2, 3, 0, a);
} /* do_6 */

void
print_vecs()
{
	t_vecs::iterator i;
	for(i = vecs.begin(); i != vecs.end(); i++) {
		printf("(%lf, %lf, %lf)\n",
			(*i).x(),
			(*i).y(),
			(*i).z());
	}
} /* print_vecs */

void 
tri_try(int n)
{
	printf("%i: %lf\n", n, (tri_merge(n) * 180.0) / M_PI);
} /* tri_try */

int
main()
{
	vecs.resize(20);
	do_6();
	print_vecs();
} /* main */
	
